#' FAQ - Is my class compatible with vctrs?
#'
#' @includeRmd man/faq/developer/reference-compatibility.Rmd description
#'
#' @name reference-faq-compatibility
NULL

#' FAQ - How does coercion work in vctrs?
#'
#' @includeRmd man/faq/developer/theory-coercion.Rmd description
#'
#' @name theory-faq-coercion
NULL

# Also see the `redirects:` section in `_pkgdown.yml`
# for `vector_recycling_rules.html`

#' FAQ - How does recycling work in vctrs and the tidyverse?
#'
#' @includeRmd man/faq/developer/theory-recycling.Rmd description
#'
#' @name theory-faq-recycling
#' @aliases vector_recycling_rules
NULL

#' FAQ - How to implement ptype2 and cast methods?
#'
#' @includeRmd man/faq/developer/howto-coercion.Rmd description
#'
#' @name howto-faq-coercion
NULL

#' FAQ - How to implement ptype2 and cast methods? (Data frames)
#'
#' @includeRmd man/faq/developer/howto-coercion-data-frame.Rmd description
#'
#' @name howto-faq-coercion-data-frame
NULL

#' FAQ - Why isn't my class treated as a vector?
#'
#' @includeRmd man/faq/developer/howto-faq-fix-scalar-type-error.Rmd description
#'
#' @name howto-faq-fix-scalar-type-error
#' @aliases howto_faq_fix_scalar_type_error
NULL
