# Python API Reference

## Database

::: vctrs.Database
    options:
      show_source: false

## Client

::: vctrs.Client
    options:
      show_source: false

## AsyncDatabase

::: vctrs.AsyncDatabase
    options:
      show_source: false

## SearchResult

::: vctrs.SearchResult
    options:
      show_source: false
