[vctrs](README.md) / GetResult

Defined in: [index.d.ts:20](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L20)

A vector and its metadata, returned by `get()`.

## Properties

| Property | Type | Description | Defined in |
| ------ | ------ | ------ | ------ |
| <a id="metadata"></a> `metadata?` | `Record`\<`string`, `any`\> | The metadata object attached to this vector, or undefined. | [index.d.ts:24](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L24) |
| <a id="vector"></a> `vector` | `number`\[\] | The stored embedding vector. | [index.d.ts:22](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L22) |
