# Semantic Search

Search over documents using sentence embeddings.

=== "Python"

    ```python
    --8<-- "examples/semantic-search/search.py"
    ```

=== "TypeScript"

    ```typescript
    --8<-- "examples/semantic-search/search.ts"
    ```

## Run it

=== "Python"

    ```bash
    pip install vctrs sentence-transformers
    python examples/semantic-search/search.py
    ```

=== "TypeScript"

    ```bash
    npm install @yang-29/vctrs @xenova/transformers
    npx tsx examples/semantic-search/search.ts
    ```
