% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-table.R
\name{table}
\alias{table}
\title{Table S3 class}
\description{
These functions help the base table class fit into the vctrs type system
by providing coercion and casting functions.
}
\keyword{internal}
