% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_as_index}
\alias{vec_as_index}
\title{Convert to an index vector}
\usage{
vec_as_index(i, n, names = NULL)
}
\arguments{
\item{i}{An index vector to convert.}

\item{n}{A single integer representing the total size of the
object that \code{i} is meant to index into.}

\item{names}{If \code{i} is a character vector, \code{names} should be a character
vector that \code{i} will be matched against to construct the index. Otherwise,
not used. The default value of \code{NULL} will result in an error
if \code{i} is a character vector.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{vec_as_index()} has been renamed to \code{\link[=vec_as_location]{vec_as_location()}} and is
deprecated as of vctrs 0.2.2.
}
\keyword{internal}
